
/*************************************************************************** 
  * Application Program: tbcas.cpp 
  * Encodes and decodes a sample baseball card 
  * using BCAS (Baseball Card Abstract Syntax) 
  */ 

#include <stdio.h> 
#include "bcas.h" 

void printStr (OssString&);    /* function for printing    * 
                  * non-PDU OssString object */ 

/* Encodes a card.  If the card is successfully encoded, decodes it. */ 

int main () 
{ 
    /* Create control table object and check the result of this operation */ 
    asn1_clean_error(); 
    bcas_Control ctl; 
    if (asn1_get_last_error()) { 
        printf("OssControl object was not created. " 
                "Error code is: %d\n", asn1_get_last_error()); 
        return 1; 
    } 

    BBCard_PDU pdu;            /* PDU object for data */ 
    BBCard_PDU pduDecoded;        /* PDU object for decoded data */ 
    int rc; 

    /* Set encoding rules to be used for encoding and decoding 
     * Use OSS_PER_ALIGNED for aligned PER, OSS_PER_UNALIGNED for 
     * unaligned PER, OSS_CER for CER, OSS_DER for DER, and OSS_XER 
     * for Basic-XER. 
     */ 
    ctl.setDecodingFlags(OSS_BER); 

    bool isXER = (ctl.getEncodingRules() == OSS_XER); 

    /* Enable debugging for both encoder and decoder */ 
    ctl.setEncodingFlags(DEBUGPDU); 
    ctl.setDecodingFlags(DEBUGPDU); 

    /* Create a copy of value notation */ 
    BBCard myCard_copy(myCard); 

    /* Set PDU object in this value */ 
    pdu.set_data(myCard_copy); 

    /* Print the input to the encoder */ 
    printf("\nThe input to the encoder ...\n\n"); 
    pdu.print(ctl); 

    EncodedBuffer encodedData;        /* a buffer for encoded data */ 

    /* Encode a card. */ 

    if (!isXER) 
    printf("\nThe encoder's trace messages ...\n"); 
    printf("\n"); 

if (rc = pdu.encode(ctl, encodedData)) { 
    /* an error occurred, print errmsg */ 
    printf("%s\n", ctl.getErrorMsg()); 
    return 1; 
    } else 
    printf("Card encoded.\n");    /* card was successfully encoded */ 

    /* Get value and length of encoded data and print as hexadecimal string */ 
    printf("The encoded data is:\n"); 
    if (!isXER) 
        ctl.printHex(encodedData.get_data(), encodedData.get_data_size()); 
    else 
        printf("\n%.*s\n", encodedData.get_data_size(), encodedData.get_data()); 

    /* Decode a card whose encoding is in encodedData.  Return non-zero 
     * for failure. */ 

    if (!isXER) 
    printf("\nThe decoder's trace messages ...\n"); 
    printf("\n"); 

if (pduDecoded.decode(ctl, encodedData)) { 
    /* an error occurred, print errmsg */ 
    printf("%s\n", ctl.getErrorMsg()); 
    return 1; 
    } else 
    printf("Card decoded.\n");    /* card was successfully decoded */ 

    /* Get data from PDU object */ 
    BBCard *myCardPtr = pduDecoded.get_data();    /* address of decoded data */ 

    /* Print out the player's batting average. */ 
    printStr(myCardPtr->get_name()); 
    printf(" of the "); 
    printStr(myCardPtr->get_team()); 
    printf(" has a batting average of %s", 
        myCardPtr->get_batting_average().get_value()); 

    /* Print the decoder's output */ 
    printf("\nOutput from the decoder ...\n\n"); 
    pduDecoded.print(ctl); 

    /* Create the new card */ 
    BBCard myFutureCard(*myCardPtr); 
    delete myCardPtr;    

    /* Change some fields */ 
    myFutureCard.set_team(BBCard::team("Mudville Ten")); 
    myFutureCard.set_age(BBCard::age(33)); 
    myFutureCard.set_batting_average(BBCard::batting_average("0.26")); 

    printf("\nChanged card is the input to the encoder ... \n"); 

    /* print the results */ 
    pdu.set_data(myFutureCard); 
    pdu.print(ctl); 

    /* Encode a card. */ 

    if (!isXER) 
    printf("\nThe encoder's trace messages ...\n"); 
    printf("\n"); 

if (pdu.encode(ctl, encodedData)) { 
    printf("%s\n", ctl.getErrorMsg()); 
    return 1; 
    } else 
    printf("Card encoded.\n"); 

    printf("The encoded data is:\n"); 
    if (!isXER) 
        ctl.printHex(encodedData.get_data(), encodedData.get_data_size()); 
    else 
        printf("\n%.*s\n", encodedData.get_data_size(), encodedData.get_data()); 

    /* Decode a card whose encoding is in encodedData. */ 

    if (!isXER) 
        printf("\nThe decoder's trace messages ...\n"); 
    printf("\n"); 

if (pduDecoded.decode(ctl, encodedData)) { 
    printf("%s\n", ctl.getErrorMsg()); 
    return 1; 
    } else 
    printf("Card decoded.\n"); 

    /* Create object for encoded data and copy there data from PDU object */ 
    BBCard myOtherCard = *pduDecoded.get_data(); 

    /* Print out the player's batting average. */ 
    printStr(myOtherCard.get_name()); 
    printf(" of the "); 
    printStr(myOtherCard.get_team()); 
    printf(" has a batting average of %s", 
        myOtherCard.get_batting_average().get_value()); 

    /* Print the decoder's output */ 
    printf("\nOutput from the decoder ...\n\n"); 
    pduDecoded.print(ctl); 

    /* Free memory allocated for storing representation object in PDU */ 
    delete pduDecoded.get_data(); 

    return 0; 
} 

void printStr (OssString& str) 
{ 
    for(unsigned i = 0; i < str.length(); i++) 
    printf("%c", str[i]);            
} 

